@@ -74,7 +74,7 @@ def signal_handler(signum, _):
 
 signal.signal(signal.SIGINT, signal_handler)
 
-def remove_repository_via_cli(line, codename, forceYes):
+def remove_ppa(line, codename, forceYes):
     if line.startswith("ppa:"):
         user, sep, ppa_name = line.split(":")[1].partition("/")
         ppa_name = ppa_name or "ppa"
@@ -100,25 +100,11 @@ def remove_repository_via_cli(line, codename, forceYes):
         except Exception as detail:
             print (_("Cannot get info about PPA: '%s'.") % detail)
 
-        (deb_line, file) = expand_ppa_line(line.strip(), codename)
-        deb_line = expand_http_line(deb_line, codename) + "\n"
-        debsrc_line = 'deb-src' + deb_line[3:] + "\n"
-
         # Remove the PPA from sources.list.d
+        (deb_line, file, key_path) = expand_ppa_line(line.strip(), codename)
         try:
-            with open(file, "r", encoding="utf-8", errors="ignore") as readfile:
-                content = readfile.readlines()
-                for line in (deb_line, debsrc_line):
-                    if line in content:
-                        content.remove(line)
-                    elif "# %s" % line in content:
-                        content.remove("# %s" % line)
-            with open(file, "w", encoding="utf-8", errors="ignore") as writefile:
-                writefile.writelines(content)
-
-            # If file no longer contains any "deb" instances, delete it as well
-            if not next((s for s in content if "deb " in s), None):
-                os.unlink(file)
+            os.unlink(file)
+            os.unlink(key_path)
         except IOError as detail:
             print (_("failed to remove PPA: '%s'") % detail)
 
@@ -142,7 +128,7 @@ def remove_repository_via_cli(line, codename, forceYes):
             print (_("failed to remove repository: '%s'") % detail)
 
 
-def add_repository_via_cli(line, codename, forceYes, use_ppas):
+def add_ppa(line, codename, forceYes, use_ppas):
 
     if line.startswith("ppa:"):
         if use_ppas != "true":
@@ -174,12 +160,12 @@ def add_repository_via_cli(line, codename, forceYes, use_ppas):
                 print(_("Unable to prompt for response.  Please run with -y"))
                 sys.exit(1)
 
-        (deb_line, file) = expand_ppa_line(line.strip(), codename)
+        (deb_line, file, key_path) = expand_ppa_line(line.strip(), codename)
         deb_line = expand_http_line(deb_line, codename)
         debsrc_line = 'deb-src' + deb_line[3:]
 
         # Add the key if not in keyring
-        add_new_key(ppa_info["signing_key_fingerprint"])
+        add_remote_key(ppa_info["signing_key_fingerprint"], path=key_path)
 
         # Add the PPA
         sources_enabled = os.path.exists("/etc/apt/sources.list.d/official-source-repositories.list")
@@ -199,29 +185,34 @@ def add_repository_via_cli(line, codename, forceYes, use_ppas):
             with open(additional_repositories_file, "a", encoding="utf-8", errors="ignore") as f:
                 f.write("%s\n" % line)
 
-def add_new_key(key):
-    # """ Add the key if not in keyring """
-    # keys = subprocess.run(["apt-key","--quiet", "adv","--with-colons", "--batch",\
-    #     "--fixed-list-mode", "--list-keys"], stdout=subprocess.PIPE,
-    #         stderr=subprocess.DEVNULL).stdout.decode().split(":")
-    # if not key in keys:
-    #     add_key_remote(key)
-    return add_key_remote(key)
-
-def add_key_remote(key):
+def add_remote_key(fingerprint, path=None):
     try:
-        proxy = []
-        if os.environ.get('http_proxy') is not None:
-            if os.environ.get('http_proxy') != "":
-                # might be an good idea to check the environment variable content for url only ....
-                # simple way could be
-                # try:
-                #     urllib.urlopen(url)
-                # except IOError:
-                #     print "Not a real URL"
-                # is unclear to me about the fault message
-                proxy=["--keyserver-options", "http-proxy=%s" % os.environ.get('http_proxy')]
-        subprocess.run(["apt-key", "adv", "--keyserver", "hkps://keyserver.ubuntu.com:443"] + proxy + ["--recv-keys", key], check=True)
+        os.system("mkdir -p /etc/apt/keyrings")
+        keyring = f"/etc/apt/keyrings/{fingerprint}.keyring"
+        tmp_keyring = f"{keyring}~"
+        key = f"/etc/apt/keyrings/{fingerprint}.gpg"
+        trusted_key = f"/etc/apt/trusted.gpg.d/{fingerprint}.gpg"
+        keyserver = "hkps://keyserver.ubuntu.com:443"
+        # Run gpg --list-keys to initiate gpg folders in ~/.gnupg
+        # otherwise gpg commands fail when we import keys
+        subprocess.run(["gpg", "--list-keys"])
+        proxy = os.environ.get('http_proxy')
+        if proxy is not None and proxy != "":
+            cmd = ["gpg", "--yes", "--honor-http-proxy", "--no-default-keyring", "--keyring", keyring, "--keyserver", keyserver, "--recv-keys", fingerprint]
+        else:
+            cmd = ["gpg", "--yes", "--no-default-keyring", "--keyring", keyring, "--keyserver", keyserver, "--recv-keys", fingerprint]
+        # import keyring
+        subprocess.run(cmd, check=True)
+        # export key
+        subprocess.run(["gpg", "--yes", "--no-default-keyring", "--keyring", keyring, "--export", "-o", key], check=True)
+        # remove keyring
+        subprocess.run(["rm", "-f", keyring, tmp_keyring])
+        # Move the key to proper place
+        # path if given (for PPAs) or globally trusted in APT dir otherwise
+        if path == None:
+            subprocess.run(["mv", key, trusted_key])
+        else:
+            subprocess.run(["mv", key, path])
     except subprocess.CalledProcessError:
         return False
     return True
@@ -278,7 +269,7 @@ def get_ppa_info_from_lp(owner_name, ppa_name, base_codename):
 
     # Make sure the PPA supports our base release
     try:
-        data = retrieve_ppa_url("http://ppa.launchpad.net/%s/%s/ubuntu/dists/%s" % (owner_name, ppa_name, base_codename))
+        data = retrieve_ppa_url("https://ppa.launchpadcontent.net/%s/%s/ubuntu/dists/%s" % (owner_name, ppa_name, base_codename))
     except PPAException as e:
         raise PPAException(e.value)
     if not data.ok:
@@ -301,10 +292,11 @@ def expand_ppa_line(abrev, distro_codename):
         ppa_name = abrev.split("/")[1]
     except IndexError:
         ppa_name = "ppa"
-    sourceslistd = "/etc/apt/sources.list.d"
-    line = "deb http://ppa.launchpad.net/%s/%s/ubuntu %s main" % (ppa_owner, ppa_name, distro_codename)
-    filename = os.path.join(sourceslistd, "%s-%s-%s.list" % (encode(ppa_owner), encode(ppa_name), distro_codename))
-    return (line, filename)
+    ppa_id = "%s-%s-%s" % (encode(ppa_owner), encode(ppa_name), distro_codename)
+    source_path = f"/etc/apt/sources.list.d/{ppa_id}.list"
+    key_path = f"/etc/apt/keyrings/{ppa_id}.gpg"
+    line = f"deb [arch=amd64 signed-by={key_path}] https://ppa.launchpadcontent.net/{ppa_owner}/{ppa_name}/ubuntu {distro_codename} main"
+    return (line, source_path, key_path)
 
 def expand_http_line(line, distro_codename):
     """
@@ -415,16 +407,19 @@ class Repository():
         self.modify_source_file(None)
 
     def get_ppa_name(self):
-        elements = self.line.split(" ")
-        name = elements[1].replace("deb-src ", "")
-        name = name.replace("deb ", "")
-        name = name.replace("http://ppa.launchpad.net/", "")
-        name = name.replace("/ubuntu", "")
-        name = name.replace("/ppa", "")
-        if self.line.startswith("deb-src"):
-            suffix = _("Sources")
-            name = "%s (%s)" % (name, suffix)
-        return "<b>%s</b>\n%s\n%s" % (name, self.line, self.file)
+        name = self.line
+        for element in self.line.split():
+            if "http://" in element or "https://" in element:
+                name = element
+                name = name.replace("http://ppa.launchpad.net/", "")
+                name = name.replace("https://ppa.launchpadcontent.net/", "")
+                name = name.replace("/ubuntu", "")
+                name = name.replace("/ppa", "")
+                if self.line.startswith("deb-src"):
+                    suffix = _("Sources")
+                    name = "%s (%s)" % (name, suffix)
+                break
+        return "<b>%s</b>\n<small>%s</small>\n<small>%s</small>" % (name, self.line, self.file)
 
     def get_repository_name(self):
         line = self.line.strip()
@@ -491,7 +486,7 @@ class ComponentSwitchBox(Gtk.Box):
 
         if widget.get_active() and os.path.exists("/etc/linuxmint/info"):
             if self.component.name == "romeo":
-                if self.application.show_confirmation_dialog(self.application._main_window, _("Linux Mint uses Romeo to publish packages which are not tested. Once these packages are tested, they are then moved to the official repositories. Unless you are participating in beta-testing, you should not enable this repository. Are you sure you want to enable Romeo?"), yes_no=True):
+                if self.application.show_confirmation_dialog(_("Linux Mint uses Romeo to publish packages which are not tested. Once these packages are tested, they are then moved to the official repositories. Unless you are participating in beta-testing, you should not enable this repository. Are you sure you want to enable Romeo?"), yes_no=True):
                     self.component.selected = widget.get_active()
                     self.application.apply_official_sources()
                 else:
@@ -1079,7 +1074,7 @@ class Application(object):
                             selected = False
                         if line.startswith("deb"):
                             repository = Repository(self, line, source_file, selected, self.base_mirror_names, self.base_name)
-                            if "ppa.launchpad" in line and self.config["general"]["use_ppas"] != "false":
+                            if "://ppa.launchpad" in line and self.config["general"]["use_ppas"] != "false":
                                 self.ppas.append(repository)
                             else:
                                 self.repositories.append(repository)
@@ -1126,15 +1121,11 @@ class Application(object):
 
     def fix_purge(self, widget):
         os.system("aptitude purge ~c -y")
-        image = Gtk.Image()
-        image.set_from_icon_name("preferences-other-symbolic", Gtk.IconSize.DIALOG)
-        self.show_confirmation_dialog(self._main_window, _("There is no more residual configuration on the system."), image, affirmation=True)
+        self.show_confirmation_dialog(_("There is no more residual configuration on the system."), affirmation=True)
 
     def fix_mergelist(self, widget):
         os.system("rm /var/lib/apt/lists/* -vrf")
-        image = Gtk.Image()
-        image.set_from_icon_name("preferences-other-symbolic", Gtk.IconSize.DIALOG)
-        self.show_confirmation_dialog(self._main_window, _("The problem was fixed. Please reload the cache."), image, affirmation=True)
+        self.show_confirmation_dialog(_("The problem was fixed. Please reload the cache."), affirmation=True)
         self.enable_reload_button()
 
     def remove_duplicates(self, widget):
@@ -1174,35 +1165,33 @@ class Application(object):
                         with open(listfile, 'w', encoding="utf-8", errors="ignore") as f:
                             for line in lines:
                                 f.write("%s\n" % line)
-        image = Gtk.Image()
-        image.set_from_icon_name("preferences-other-symbolic", Gtk.IconSize.DIALOG)
+
         if found_duplicates:
-            self.show_confirmation_dialog(self._main_window, _("Duplicate entries were removed. Please reload the cache."), image, affirmation=True)
+            self.show_confirmation_dialog(_("Duplicate entries were removed. Please reload the cache."), affirmation=True)
             self.enable_reload_button()
             self.read_source_lists()
             self.refresh_ppa_model()
             self.refresh_repository_model()
         else:
-            self.show_confirmation_dialog(self._main_window, _("No duplicate entries were found."), image, affirmation=True)
+            self.show_confirmation_dialog(_("No duplicate entries were found."), affirmation=True)
 
     def fix_missing_keys(self, widget):
-        image = Gtk.Image()
-        image.set_from_icon_name("dialog-password-symbolic", Gtk.IconSize.DIALOG)
-
         #get paths from apt
         apt_pkg.init()
         trusted = apt_pkg.config.find_file("Dir::Etc::trusted")
         trustedparts = apt_pkg.config.find_dir("Dir::Etc::trustedparts")
         lists = apt_pkg.config.find_dir("Dir::State::lists")
         if not os.path.isfile(trusted) or not os.path.isdir(trustedparts) or not os.path.isdir(lists):
-            self.show_confirmation_dialog(self._main_window,
-                _("Error with your APT configuration, you may have to reload the cache first."),
-                image, affirmation=True)
+            self.show_confirmation_dialog(_("Error with your APT configuration, you may have to reload the cache first."), affirmation=True)
             return
 
         self._main_window.get_window().set_cursor(Gdk.Cursor(Gdk.CursorType.WATCH))
         Gdk.flush()
 
+        # Run gpg --list-keys to initiate gpg folders in ~/.gnupg
+        # otherwise gpg commands fail when we import keys
+        subprocess.run(["gpg", "--list-keys"])
+
         cmd_stub = ["gpg", "--no-default-keyring", "--no-options"]
         keyrings = [trusted] + glob.glob("%s*.gpg" % trustedparts)
         for keyring in keyrings:
@@ -1255,6 +1244,20 @@ class Application(object):
         r = re.compile(r"^gpg\:\s+using \S+ key (.+)$", re.MULTILINE | re.IGNORECASE)
         # try to verify all repository lists using gpg
         for repository in repositories:
+            # if the repository is "signed-by", just check that the key file is present
+            uri = repository.uri
+            if uri.endswith("/"):
+                uri = uri[:-1]
+            output = subprocess.getoutput(f"inxi -r | grep {uri}")
+            key_path = None
+            if "signed-by" in output:
+                key_path = output.split("signed-by=")[1].split("]")[0]
+                print(f"{uri} is signed by {key_path}")
+                if os.path.exists(key_path):
+                    print("  Key found.")
+                    continue
+                else:
+                    print("  Key is missing.")
             if repository.path.endswith("_InRelease"):
                 command = cmd_stub + (["--verify", repository.path])
             else:
@@ -1269,7 +1272,7 @@ class Application(object):
                     key = r.search(message).group(1)
                     key = re.sub(r"\s", "", key)
                     # get key from keyserver
-                    success = add_new_key(key)
+                    success = add_remote_key(key, path=key_path)
                     if not success:
                         raise ValueError("Retrieving key %s failed" % key)
                     repository.added = True
@@ -1312,9 +1315,9 @@ class Application(object):
                 msg += "\n%s" % _("Please reload the cache.")
                 self.load_keys()
                 self.enable_reload_button()
-            self.show_confirmation_dialog(self._main_window, msg, image, affirmation=True)
+            self.show_confirmation_dialog(msg, affirmation=True)
         else:
-            self.show_confirmation_dialog(self._main_window, _("No missing keys were found."), image, affirmation=True)
+            self.show_confirmation_dialog(_("No missing keys were found."), affirmation=True)
 
     def load_keys(self):
         self.keys = []
@@ -1364,22 +1367,14 @@ class Application(object):
         dialog.destroy()
 
     def fetch_key(self, widget):
-        image = Gtk.Image()
-        image.set_from_icon_name("dialog-password-symbolic", Gtk.IconSize.DIALOG)
-        fingerprint = self.show_entry_dialog(self._main_window, _("Please enter the fingerprint of the public key you want to download from keyserver.ubuntu.com:"), "", image)
+        fingerprint = self.show_entry_dialog(_("Please enter the fingerprint of the public key you want to download from keyserver.ubuntu.com:"), "")
         if fingerprint is not None:
-            add_key_remote(fingerprint)
+            add_remote_key(fingerprint)
             self.load_keys()
             self.enable_reload_button()
 
-    def add_new_key(self, key):
-        add_new_key(key)
-        self.load_keys()
-
     def remove_key(self, widget):
-        image = Gtk.Image()
-        image.set_from_icon_name("dialog-password-symbolic", Gtk.IconSize.DIALOG)
-        if (self.show_confirmation_dialog(self._main_window, _("Are you sure you want to permanently remove the selected keys?"), image, yes_no=True)):
+        if (self.show_confirmation_dialog(_("Are you sure you want to permanently remove the selected keys?"), yes_no=True)):
             selection = self._keys_treeview.get_selection()
             (model, indexes) = selection.get_selected_rows()
             iters = []
@@ -1395,8 +1390,6 @@ class Application(object):
         self.builder.get_object("button_keys_remove").set_sensitive(selection_count >= 1)
 
     def add_ppa(self, widget):
-        image = Gtk.Image()
-        image.set_from_icon_name("process-stop-symbolic", Gtk.IconSize.DIALOG)
         default_line = "ppa:ppa-owner/ppa-name"
         start_line = default_line
         clipboard_text = self.get_clipboard_text("ppa")
@@ -1405,7 +1398,7 @@ class Application(object):
         if clipboard_text is not None:
             start_line = clipboard_text
 
-        line = self.show_entry_dialog(self._main_window, _("Please enter the name or the URL of the PPA you want to add:"), start_line, image)
+        line = self.show_entry_dialog(_("Please enter the name or the URL of the PPA you want to add:"), start_line)
         if line:
             # If the user pasted the launchpad URL, parse that into a ppa: line
             if line.startswith("https://launchpad.net/"):
@@ -1419,7 +1412,7 @@ class Application(object):
                 ppa_name = ppa_name or "ppa"
                 ppa_info = get_ppa_info_from_lp(user, ppa_name, self.config["general"]["base_codename"])
             except Exception as error_msg:
-                self.show_error_dialog(self._main_window, error_msg)
+                self.show_error_dialog(error_msg)
                 return
 
             image = Gtk.Image()
@@ -1427,13 +1420,14 @@ class Application(object):
             info_text = "%s\n\n%s\n\n%s\n\n%s" % (line,
                 self.format_string(ppa_info["displayname"]),
                 self.format_string(ppa_info["description"]), str(ppa_info["web_link"]))
-            if self.show_confirm_ppa_dialog(self._main_window, info_text):
-                (deb_line, file) = expand_ppa_line(line.strip(), self.config["general"]["base_codename"])
+            if self.show_confirm_ppa_dialog(info_text):
+                (deb_line, file, key_path) = expand_ppa_line(line.strip(), self.config["general"]["base_codename"])
                 deb_line = expand_http_line(deb_line, self.config["general"]["base_codename"])
                 debsrc_line = 'deb-src' + deb_line[3:]
 
                 # Add the key if not in keyring
-                self.add_new_key(ppa_info["signing_key_fingerprint"])
+                add_remote_key(ppa_info["signing_key_fingerprint"], path=key_path)
+                self.load_keys()
 
                 # Add the PPA in sources.list.d
                 sources_enabled = self.builder.get_object("source_code_switch").get_active()
@@ -1470,13 +1464,13 @@ class Application(object):
         (model, indexes) = selection.get_selected_rows()
         iter = model.get_iter(indexes[0])
         repository = model.get(iter, 0)[0]
-        url = self.show_entry_dialog(self._main_window, _("Edit the URL of the PPA"), repository.line)
+        url = self.show_entry_dialog(_("Edit the URL of the PPA"), repository.line)
         if url is not None:
             repository.edit(url)
             model.set_value(iter, 2, repository.get_ppa_name())
 
     def remove_ppa(self, widget):
-        if (self.show_confirmation_dialog(self._main_window, _("Are you sure you want to permanently remove the selected PPAs?"), yes_no=True)):
+        if (self.show_confirmation_dialog(_("Are you sure you want to permanently remove the selected PPAs?"), yes_no=True)):
             selection = self._ppa_treeview.get_selection()
             (model, indexes) = selection.get_selected_rows()
             iters = []
@@ -1487,6 +1481,15 @@ class Application(object):
                 model.remove(iter)
                 repository.delete()
                 self.ppas.remove(repository)
+                # If the source path was deleted, also delete the key path
+                if not os.path.exists(repository.file):
+                    print(f"{repository.file} deleted")
+                    path = repository.file
+                    path = path.replace("/etc/apt/sources.list.d/", "/etc/apt/keyrings/")
+                    path = path.replace(".list", ".gpg")
+                    if os.path.exists(path):
+                        os.unlink(path)
+                        print(f"{path} deleted")
 
     def ppa_selected(self, selection):
         selection_count = selection.count_selected_rows()
@@ -1500,7 +1503,7 @@ class Application(object):
                 iter = model.get_iter(indexes[0])
                 repository = model.get_value(iter, 0)
                 ppa_name = model.get_value(iter, 2)
-                if repository.selected and repository.line.startswith("deb http://ppa.launchpad.net"):
+                if repository.selected and "://ppa.launchpad" in repository.line:
                     self.builder.get_object("button_ppa_examine").set_sensitive(True)
             except Exception as detail:
                 print (detail)
@@ -1510,23 +1513,31 @@ class Application(object):
 
     def examine_ppa(self, widget):
         try:
+            codename = self.config["general"]["base_codename"]
+            arch = subprocess.getoutput("dpkg --print-architecture")
             selection = self._ppa_treeview.get_selection()
             (model, indexes) = selection.get_selected_rows()
             iter = model.get_iter(indexes[0])
             repository = model.get_value(iter, 0)
             ppa_name = model.get_value(iter, 2)
-            if repository.selected and repository.line.startswith("deb http://ppa.launchpad.net"):
-                line = repository.line.split()[1].replace("http://ppa.launchpad.net/", "")
-                if line.endswith("/ubuntu"):
-                    line = line[:-7]
-                    ppa_owner, ppa_name = line.split("/")
-                    architecture = subprocess.getoutput("dpkg --print-architecture")
-                    ppa_file = "/var/lib/apt/lists/ppa.launchpad.net_%s_%s_ubuntu_dists_%s_main_binary-%s_Packages" % (ppa_owner, ppa_name, self.config["general"]["base_codename"], architecture)
-                    if os.path.exists(ppa_file):
-                        os.system("/usr/lib/linuxmint/mintSources/ppa_browser.py %s %s %s &" % (self.config["general"]["base_codename"], ppa_owner, ppa_name))
-                    else:
-                        print ("%s not found!" % ppa_file)
-                        self.show_error_dialog(self._main_window, _("The content of this PPA is not available. Please refresh the cache and try again."))
+            if repository.selected and "://ppa.launchpad" in repository.line:
+                elements = repository.line.split()
+                for element in repository.line.split():
+                    if element.startswith("http") and element.endswith("/ubuntu"):
+                        line = element
+                        line = line.replace("http://ppa.launchpad.net/", "")
+                        line = line.replace("https://ppa.launchpadcontent.net/", "")
+                        line = line[:-7]
+                        ppa_owner, ppa_name = line.split("/")
+                        ppa_file = f"/var/lib/apt/lists/ppa.launchpadcontent.net_{ppa_owner}_{ppa_name}_ubuntu_dists_{codename}_main_binary-{arch}_Packages"
+                        legacy_ppa_file = f"/var/lib/apt/lists/ppa.launchpad.net_{ppa_owner}_{ppa_name}_ubuntu_dists_{codename}_main_binary-{arch}_Packages"
+                        if os.path.exists(ppa_file):
+                            subprocess.Popen(["/usr/lib/linuxmint/mintSources/ppa_browser.py", codename, ppa_file, ppa_owner, ppa_name])
+                        elif os.path.exists(legacy_ppa_file):
+                            subprocess.Popen(["/usr/lib/linuxmint/mintSources/ppa_browser.py", codename, legacy_ppa_file, ppa_owner, ppa_name])
+                        else:
+                            print ("%s not found!" % ppa_file)
+                            self.show_error_dialog(_("The content of this PPA is not available. Please refresh the cache and try again."))
         except Exception as detail:
             print (detail)
 
@@ -1536,8 +1547,6 @@ class Application(object):
         self.builder.get_object("button_repository_remove").set_sensitive(selection_count >= 1)
 
     def add_repository(self, widget):
-        image = Gtk.Image()
-        image.set_from_icon_name("network-workgroup-symbolic", Gtk.IconSize.DIALOG)
         start_line = ""
         default_line = "deb http://packages.domain.com/ %s main" % self.config["general"]["base_codename"]
         clipboard_text = self.get_clipboard_text("deb")
@@ -1546,12 +1555,12 @@ class Application(object):
         else:
             start_line = default_line
 
-        line = self.show_entry_dialog(self._main_window, _("Please enter the name of the repository you want to add:"), start_line, image)
+        line = self.show_entry_dialog(_("Please enter the name of the repository you want to add:"), start_line)
         if not line or line == default_line:
             return
         line = expand_http_line(line, self.config["general"]["base_codename"])
         if repo_malformed(line):
-            self.show_confirmation_dialog(self._main_window, _("Malformed input, repository not added."), image, affirmation=True)
+            self.show_confirmation_dialog(_("Malformed input, repository not added."), affirmation=True)
         else:
             if not repo_exists(line):
                 # Add the repository in sources.list.d
@@ -1563,20 +1572,20 @@ class Application(object):
                 tree_iter = self._repository_model.append((repository, repository.selected, repository.get_repository_name()))
                 self.enable_reload_button()
             else:
-                self.show_confirmation_dialog(self._main_window, _("This repository is already configured, you cannot add it a second time."), image, affirmation=True)
+                self.show_confirmation_dialog(_("This repository is already configured, you cannot add it a second time."), affirmation=True)
 
     def edit_repository(self, widget):
         selection = self._repository_treeview.get_selection()
         (model, indexes) = selection.get_selected_rows()
         iter = model.get_iter(indexes[0])
         repository = model.get(iter, 0)[0]
-        url = self.show_entry_dialog(self._main_window, _("Edit the URL of the repository"), repository.line)
+        url = self.show_entry_dialog(_("Edit the URL of the repository"), repository.line)
         if url is not None:
             repository.edit(url)
             model.set_value(iter, 2, repository.get_repository_name())
 
     def remove_repository(self, widget):
-        if (self.show_confirmation_dialog(self._main_window, _("Are you sure you want to permanently remove the selected repositories?"), yes_no=True)):
+        if (self.show_confirmation_dialog(_("Are you sure you want to permanently remove the selected repositories?"), yes_no=True)):
             selection = self._repository_treeview.get_selection()
             (model, indexes) = selection.get_selected_rows()
             iters = []
@@ -1588,7 +1597,7 @@ class Application(object):
                 repository.delete()
                 self.repositories.remove(repository)
 
-    def show_confirmation_dialog(self, parent, message, image=None, affirmation=None, yes_no=False):
+    def show_confirmation_dialog(self, message, affirmation=None, yes_no=False):
         buttons = Gtk.ButtonsType.OK_CANCEL
         default_button = Gtk.ResponseType.OK
         confirmation_button = Gtk.ResponseType.OK
@@ -1598,20 +1607,17 @@ class Application(object):
             confirmation_button = Gtk.ResponseType.YES
 
         if affirmation is None:
-            d = Gtk.MessageDialog(parent,
-                              Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                              Gtk.MessageType.WARNING,
-                              buttons,
-                              message)
+            d = Gtk.MessageDialog(parent=self._main_window,
+                              message_type=Gtk.MessageType.WARNING,
+                              buttons=buttons,
+                              text=message,
+                              modal=True)
         else:
-            d = Gtk.MessageDialog(parent,
-                              Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                              Gtk.MessageType.INFO,
-                              Gtk.ButtonsType.OK,
-                              message)
-        if image is not None:
-            image.show()
-            d.set_image(image)
+            d = Gtk.MessageDialog(parent=self._main_window,
+                              message_type=Gtk.MessageType.INFO,
+                              buttons=Gtk.ButtonsType.OK,
+                              text=message,
+                              modal=True)
 
         d.set_default_response(default_button)
         r = d.run()
@@ -1621,7 +1627,7 @@ class Application(object):
         else:
             return False
 
-    def show_confirm_ppa_dialog(self, parent, message):
+    def show_confirm_ppa_dialog(self, message):
         b = Gtk.TextBuffer()
         b.set_text(message)
         t =  Gtk.TextView()
@@ -1632,7 +1638,7 @@ class Application(object):
         s.set_shadow_type(Gtk.ShadowType.OUT)
         default_button = Gtk.ResponseType.ACCEPT
         confirmation_button = Gtk.ResponseType.ACCEPT
-        d = Gtk.Dialog(None, parent,
+        d = Gtk.Dialog(None, self._main_window,
                        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
                        (_("Cancel"), Gtk.ResponseType.REJECT,
                        _("OK"), Gtk.ResponseType.ACCEPT))
@@ -1650,17 +1656,12 @@ class Application(object):
         else:
             return False
 
-    def show_error_dialog(self, parent, message, image=None):
-        d = Gtk.MessageDialog(parent,
-                              Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                              Gtk.MessageType.ERROR,
-                              Gtk.ButtonsType.OK,
-                              message)
-
-        if image is not None:
-            image.show()
-            d.set_image(image)
-
+    def show_error_dialog(self, message):
+        d = Gtk.MessageDialog(parent=self._main_window,
+                              message_type=Gtk.MessageType.ERROR,
+                              buttons=Gtk.ButtonsType.OK,
+                              text=message,
+                              modal=True)
         d.set_default_response(Gtk.ResponseType.OK)
         r = d.run()
         d.destroy()
@@ -1669,17 +1670,12 @@ class Application(object):
         else:
             return False
 
-    def show_entry_dialog(self, parent, message, default='', image=None):
-        d = Gtk.MessageDialog(parent,
-                              Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                              Gtk.MessageType.QUESTION,
-                              Gtk.ButtonsType.OK_CANCEL,
-                              message)
-
-        if image is not None:
-            image.show()
-            d.set_image(image)
-
+    def show_entry_dialog(self, message, default=''):
+        d = Gtk.MessageDialog(parent=self._main_window,
+                              message_type=Gtk.MessageType.OTHER,
+                              buttons=Gtk.ButtonsType.OK_CANCEL,
+                              text=message,
+                              modal=True)
         entry = Gtk.Entry()
         entry.set_text(default)
         entry.set_margin_start(6)
@@ -1779,9 +1775,15 @@ class Application(object):
                 self._info_box.pack_start(infobar, True, True, 0)
 
             self._info_revealer.set_reveal_child(True)
+            self.builder.get_object("ppas_page").set_margin_bottom(50)
+            self.builder.get_object("additional_repositories_page").set_margin_bottom(50)
+            self.builder.get_object("authentication_keys_page").set_margin_bottom(50)
 
     def _on_infobar_response(self, infobar, response_id):
         self._info_revealer.set_reveal_child(False)
+        self.builder.get_object("ppas_page").set_margin_bottom(12)
+        self.builder.get_object("additional_repositories_page").set_margin_bottom(12)
+        self.builder.get_object("authentication_keys_page").set_margin_bottom(12)
 
         if response_id == Gtk.ResponseType.OK:
             self.apt.update_cache()
@@ -1965,9 +1967,9 @@ if __name__ == "__main__":
         codename = config_parser.get("general", "base_codename")
         use_ppas = config_parser.get("general", "use_ppas")
         if "-r" in args:
-            remove_repository_via_cli(ppa_line, codename, "-y" in args)
+            remove_ppa(ppa_line, codename, "-y" in args)
         else:
-            add_repository_via_cli(ppa_line, codename, "-y" in args, use_ppas)
+            add_ppa(ppa_line, codename, "-y" in args, use_ppas)
     else:
         Application().run()
 
